
/*
 * ECMWF Air Quality Forecast - A Weather Source API
 *
 * Daily and hourly air quality forecast data from the ECMWF model for latitude/longitude points, ZIP/Postal Codes, and Designated Market Areas. Hourly forecast is up to 5 days out from current day. Daily forecast is up to 4 days out from current day. The Weather Source ECMWF Air Quality Forecast API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.    **IMPORTANT:** To authenticate to the Weather Source ECMWF Air Quality Forecast API, each request must contain your API key set to a custom header named `X-API-KEY`. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key `C0W60UOFRML47ytbXk4xlLBfv` to access example locations found in this documentation.      When using the \\\"Try it out\\\" capability in the documentation, the API key should be added by clicking the green \\\"Authorize\\\" button at the top of the page. 
 *
 * API version: 2.0.0
 * Contact: support@weathersource.com
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */
package swagger

import (
	"context"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"
	"fmt"
	"github.com/antihax/optional"
	"time"
)

// Linger please
var (
	_ context.Context
)

type PostcodesApiService service
/*
PostcodesApiService A day of ECMWF Air Quality Forecast data for a postcode.
Returns a single day of ECMWF Air Quality Forecast data for a postcode. The provided date must exist within the current valid 9-day forecast range.  **IMPORTANT:** To authenticate to the Weather Source ECMWF Air Quality Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
 * @param ctx context.Context - for authentication, logging, cancellation, deadlines, tracing, etc. Passed from http.Request or context.Background().
 * @param postcode A postcode string.
 * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
 * @param date An date formatted as a RFC3339 date value.
 * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* 
 * @param optional nil or *PostcodesApiPostcodesPostcodecountryCodeDaysDateGetOpts - Optional Parameters:
     * @param "UnitScale" (optional.String) -  The unit scale for returned values.&lt;br&gt;
@return EcmwfAirQualityForecastPostcodeDayObj
*/

type PostcodesApiPostcodesPostcodecountryCodeDaysDateGetOpts struct {
    UnitScale optional.String
}

func (a *PostcodesApiService) PostcodesPostcodecountryCodeDaysDateGet(ctx context.Context, postcode string, countryCode string, date string, fields []string, localVarOptionals *PostcodesApiPostcodesPostcodecountryCodeDaysDateGetOpts) (EcmwfAirQualityForecastPostcodeDayObj, *http.Response, error) {
	var (
		localVarHttpMethod = strings.ToUpper("Get")
		localVarPostBody   interface{}
		localVarFileName   string
		localVarFileBytes  []byte
		localVarReturnValue EcmwfAirQualityForecastPostcodeDayObj
	)

	// create path and map variables
	localVarPath := a.client.cfg.BasePath + "/postcodes/{postcode},{countryCode}/days/{date}"
	localVarPath = strings.Replace(localVarPath, "{"+"postcode"+"}", fmt.Sprintf("%v", postcode), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"countryCode"+"}", fmt.Sprintf("%v", countryCode), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"date"+"}", fmt.Sprintf("%v", date), -1)

	localVarHeaderParams := make(map[string]string)
	localVarQueryParams := url.Values{}
	localVarFormParams := url.Values{}

	localVarQueryParams.Add("fields", parameterToString(fields, "csv"))
	if localVarOptionals != nil && localVarOptionals.UnitScale.IsSet() {
		localVarQueryParams.Add("unitScale", parameterToString(localVarOptionals.UnitScale.Value(), ""))
	}
	// to determine the Content-Type header
	localVarHttpContentTypes := []string{}

	// set Content-Type header
	localVarHttpContentType := selectHeaderContentType(localVarHttpContentTypes)
	if localVarHttpContentType != "" {
		localVarHeaderParams["Content-Type"] = localVarHttpContentType
	}

	// to determine the Accept header
	localVarHttpHeaderAccepts := []string{"application/json"}

	// set Accept header
	localVarHttpHeaderAccept := selectHeaderAccept(localVarHttpHeaderAccepts)
	if localVarHttpHeaderAccept != "" {
		localVarHeaderParams["Accept"] = localVarHttpHeaderAccept
	}
	if ctx != nil {
		// API Key Authentication
		if auth, ok := ctx.Value(ContextAPIKey).(APIKey); ok {
			var key string
			if auth.Prefix != "" {
				key = auth.Prefix + " " + auth.Key
			} else {
				key = auth.Key
			}
			localVarHeaderParams["X-API-KEY"] = key
			
		}
	}
	r, err := a.client.prepareRequest(ctx, localVarPath, localVarHttpMethod, localVarPostBody, localVarHeaderParams, localVarQueryParams, localVarFormParams, localVarFileName, localVarFileBytes)
	if err != nil {
		return localVarReturnValue, nil, err
	}

	localVarHttpResponse, err := a.client.callAPI(r)
	if err != nil || localVarHttpResponse == nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	localVarBody, err := ioutil.ReadAll(localVarHttpResponse.Body)
	localVarHttpResponse.Body.Close()
	if err != nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	if localVarHttpResponse.StatusCode < 300 {
		// If we succeed, return the data, otherwise pass on to decode error.
		err = a.client.decode(&localVarReturnValue, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
		if err == nil { 
			return localVarReturnValue, localVarHttpResponse, err
		}
	}

	if localVarHttpResponse.StatusCode >= 300 {
		newErr := GenericSwaggerError{
			body: localVarBody,
			error: localVarHttpResponse.Status,
		}
		if localVarHttpResponse.StatusCode == 200 {
			var v EcmwfAirQualityForecastPostcodeDayObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		if localVarHttpResponse.StatusCode == 0 {
			var v ErrorObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		return localVarReturnValue, localVarHttpResponse, newErr
	}

	return localVarReturnValue, localVarHttpResponse, nil
}
/*
PostcodesApiService A contiguous range of daily ECMWF Air Quality Forecast data for a postcode.
Returns a contiguous range of daily ECMWF Air Quality Forecast data for a postcode. The provided dates must exist within the current valid 9-day forecast range.  **IMPORTANT:** To authenticate to the Weather Source ECMWF Air Quality Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
 * @param ctx context.Context - for authentication, logging, cancellation, deadlines, tracing, etc. Passed from http.Request or context.Background().
 * @param postcode A postcode string.
 * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
 * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd.
 * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart.
 * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* 
 * @param optional nil or *PostcodesApiPostcodesPostcodecountryCodeDaysDateStartdateEndGetOpts - Optional Parameters:
     * @param "UnitScale" (optional.String) -  The unit scale for returned values.&lt;br&gt;
@return EcmwfAirQualityForecastPostcodeDayObj
*/

type PostcodesApiPostcodesPostcodecountryCodeDaysDateStartdateEndGetOpts struct {
    UnitScale optional.String
}

func (a *PostcodesApiService) PostcodesPostcodecountryCodeDaysDateStartdateEndGet(ctx context.Context, postcode string, countryCode string, dateStart string, dateEnd string, fields []string, localVarOptionals *PostcodesApiPostcodesPostcodecountryCodeDaysDateStartdateEndGetOpts) (EcmwfAirQualityForecastPostcodeDayObj, *http.Response, error) {
	var (
		localVarHttpMethod = strings.ToUpper("Get")
		localVarPostBody   interface{}
		localVarFileName   string
		localVarFileBytes  []byte
		localVarReturnValue EcmwfAirQualityForecastPostcodeDayObj
	)

	// create path and map variables
	localVarPath := a.client.cfg.BasePath + "/postcodes/{postcode},{countryCode}/days/{dateStart},{dateEnd}"
	localVarPath = strings.Replace(localVarPath, "{"+"postcode"+"}", fmt.Sprintf("%v", postcode), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"countryCode"+"}", fmt.Sprintf("%v", countryCode), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"dateStart"+"}", fmt.Sprintf("%v", dateStart), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"dateEnd"+"}", fmt.Sprintf("%v", dateEnd), -1)

	localVarHeaderParams := make(map[string]string)
	localVarQueryParams := url.Values{}
	localVarFormParams := url.Values{}

	localVarQueryParams.Add("fields", parameterToString(fields, "csv"))
	if localVarOptionals != nil && localVarOptionals.UnitScale.IsSet() {
		localVarQueryParams.Add("unitScale", parameterToString(localVarOptionals.UnitScale.Value(), ""))
	}
	// to determine the Content-Type header
	localVarHttpContentTypes := []string{}

	// set Content-Type header
	localVarHttpContentType := selectHeaderContentType(localVarHttpContentTypes)
	if localVarHttpContentType != "" {
		localVarHeaderParams["Content-Type"] = localVarHttpContentType
	}

	// to determine the Accept header
	localVarHttpHeaderAccepts := []string{"application/json"}

	// set Accept header
	localVarHttpHeaderAccept := selectHeaderAccept(localVarHttpHeaderAccepts)
	if localVarHttpHeaderAccept != "" {
		localVarHeaderParams["Accept"] = localVarHttpHeaderAccept
	}
	if ctx != nil {
		// API Key Authentication
		if auth, ok := ctx.Value(ContextAPIKey).(APIKey); ok {
			var key string
			if auth.Prefix != "" {
				key = auth.Prefix + " " + auth.Key
			} else {
				key = auth.Key
			}
			localVarHeaderParams["X-API-KEY"] = key
			
		}
	}
	r, err := a.client.prepareRequest(ctx, localVarPath, localVarHttpMethod, localVarPostBody, localVarHeaderParams, localVarQueryParams, localVarFormParams, localVarFileName, localVarFileBytes)
	if err != nil {
		return localVarReturnValue, nil, err
	}

	localVarHttpResponse, err := a.client.callAPI(r)
	if err != nil || localVarHttpResponse == nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	localVarBody, err := ioutil.ReadAll(localVarHttpResponse.Body)
	localVarHttpResponse.Body.Close()
	if err != nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	if localVarHttpResponse.StatusCode < 300 {
		// If we succeed, return the data, otherwise pass on to decode error.
		err = a.client.decode(&localVarReturnValue, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
		if err == nil { 
			return localVarReturnValue, localVarHttpResponse, err
		}
	}

	if localVarHttpResponse.StatusCode >= 300 {
		newErr := GenericSwaggerError{
			body: localVarBody,
			error: localVarHttpResponse.Status,
		}
		if localVarHttpResponse.StatusCode == 200 {
			var v EcmwfAirQualityForecastPostcodeDayObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		if localVarHttpResponse.StatusCode == 0 {
			var v ErrorObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		return localVarReturnValue, localVarHttpResponse, newErr
	}

	return localVarReturnValue, localVarHttpResponse, nil
}
/*
PostcodesApiService The complete daily ECMWF Air Quality Forecast for a postcode.
Returns the complete set of ECMWF Air Quality Forecast data for a postcode.  *Alternatively using a query that specificies just the days required for your purposes will improve the query response time.*  **IMPORTANT:** To authenticate to the Weather Source ECMWF Air Quality Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
 * @param ctx context.Context - for authentication, logging, cancellation, deadlines, tracing, etc. Passed from http.Request or context.Background().
 * @param postcode A postcode string.
 * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
 * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* 
 * @param optional nil or *PostcodesApiPostcodesPostcodecountryCodeDaysGetOpts - Optional Parameters:
     * @param "UnitScale" (optional.String) -  The unit scale for returned values.&lt;br&gt;
@return EcmwfAirQualityForecastPostcodeDayObj
*/

type PostcodesApiPostcodesPostcodecountryCodeDaysGetOpts struct {
    UnitScale optional.String
}

func (a *PostcodesApiService) PostcodesPostcodecountryCodeDaysGet(ctx context.Context, postcode string, countryCode string, fields []string, localVarOptionals *PostcodesApiPostcodesPostcodecountryCodeDaysGetOpts) (EcmwfAirQualityForecastPostcodeDayObj, *http.Response, error) {
	var (
		localVarHttpMethod = strings.ToUpper("Get")
		localVarPostBody   interface{}
		localVarFileName   string
		localVarFileBytes  []byte
		localVarReturnValue EcmwfAirQualityForecastPostcodeDayObj
	)

	// create path and map variables
	localVarPath := a.client.cfg.BasePath + "/postcodes/{postcode},{countryCode}/days"
	localVarPath = strings.Replace(localVarPath, "{"+"postcode"+"}", fmt.Sprintf("%v", postcode), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"countryCode"+"}", fmt.Sprintf("%v", countryCode), -1)

	localVarHeaderParams := make(map[string]string)
	localVarQueryParams := url.Values{}
	localVarFormParams := url.Values{}

	localVarQueryParams.Add("fields", parameterToString(fields, "csv"))
	if localVarOptionals != nil && localVarOptionals.UnitScale.IsSet() {
		localVarQueryParams.Add("unitScale", parameterToString(localVarOptionals.UnitScale.Value(), ""))
	}
	// to determine the Content-Type header
	localVarHttpContentTypes := []string{}

	// set Content-Type header
	localVarHttpContentType := selectHeaderContentType(localVarHttpContentTypes)
	if localVarHttpContentType != "" {
		localVarHeaderParams["Content-Type"] = localVarHttpContentType
	}

	// to determine the Accept header
	localVarHttpHeaderAccepts := []string{"application/json"}

	// set Accept header
	localVarHttpHeaderAccept := selectHeaderAccept(localVarHttpHeaderAccepts)
	if localVarHttpHeaderAccept != "" {
		localVarHeaderParams["Accept"] = localVarHttpHeaderAccept
	}
	if ctx != nil {
		// API Key Authentication
		if auth, ok := ctx.Value(ContextAPIKey).(APIKey); ok {
			var key string
			if auth.Prefix != "" {
				key = auth.Prefix + " " + auth.Key
			} else {
				key = auth.Key
			}
			localVarHeaderParams["X-API-KEY"] = key
			
		}
	}
	r, err := a.client.prepareRequest(ctx, localVarPath, localVarHttpMethod, localVarPostBody, localVarHeaderParams, localVarQueryParams, localVarFormParams, localVarFileName, localVarFileBytes)
	if err != nil {
		return localVarReturnValue, nil, err
	}

	localVarHttpResponse, err := a.client.callAPI(r)
	if err != nil || localVarHttpResponse == nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	localVarBody, err := ioutil.ReadAll(localVarHttpResponse.Body)
	localVarHttpResponse.Body.Close()
	if err != nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	if localVarHttpResponse.StatusCode < 300 {
		// If we succeed, return the data, otherwise pass on to decode error.
		err = a.client.decode(&localVarReturnValue, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
		if err == nil { 
			return localVarReturnValue, localVarHttpResponse, err
		}
	}

	if localVarHttpResponse.StatusCode >= 300 {
		newErr := GenericSwaggerError{
			body: localVarBody,
			error: localVarHttpResponse.Status,
		}
		if localVarHttpResponse.StatusCode == 200 {
			var v EcmwfAirQualityForecastPostcodeDayObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		if localVarHttpResponse.StatusCode == 0 {
			var v ErrorObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		return localVarReturnValue, localVarHttpResponse, newErr
	}

	return localVarReturnValue, localVarHttpResponse, nil
}
/*
PostcodesApiService The complete hourly ECMWF Air Quality Forecast for a postcode.
Returns the complete set of ECMWF Air Quality Forecast data for a postcode.  *Alternatively using a query that specificies just the hours required for your purposes will improve the query response time.*  **IMPORTANT:** To authenticate to the Weather Source ECMWF Air Quality Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
 * @param ctx context.Context - for authentication, logging, cancellation, deadlines, tracing, etc. Passed from http.Request or context.Background().
 * @param postcode A postcode string.
 * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
 * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* 
 * @param optional nil or *PostcodesApiPostcodesPostcodecountryCodeHoursGetOpts - Optional Parameters:
     * @param "UnitScale" (optional.String) -  The unit scale for returned values.&lt;br&gt;
@return EcmwfAirQualityForecastPostcodeHourObj
*/

type PostcodesApiPostcodesPostcodecountryCodeHoursGetOpts struct {
    UnitScale optional.String
}

func (a *PostcodesApiService) PostcodesPostcodecountryCodeHoursGet(ctx context.Context, postcode string, countryCode string, fields []string, localVarOptionals *PostcodesApiPostcodesPostcodecountryCodeHoursGetOpts) (EcmwfAirQualityForecastPostcodeHourObj, *http.Response, error) {
	var (
		localVarHttpMethod = strings.ToUpper("Get")
		localVarPostBody   interface{}
		localVarFileName   string
		localVarFileBytes  []byte
		localVarReturnValue EcmwfAirQualityForecastPostcodeHourObj
	)

	// create path and map variables
	localVarPath := a.client.cfg.BasePath + "/postcodes/{postcode},{countryCode}/hours"
	localVarPath = strings.Replace(localVarPath, "{"+"postcode"+"}", fmt.Sprintf("%v", postcode), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"countryCode"+"}", fmt.Sprintf("%v", countryCode), -1)

	localVarHeaderParams := make(map[string]string)
	localVarQueryParams := url.Values{}
	localVarFormParams := url.Values{}

	localVarQueryParams.Add("fields", parameterToString(fields, "csv"))
	if localVarOptionals != nil && localVarOptionals.UnitScale.IsSet() {
		localVarQueryParams.Add("unitScale", parameterToString(localVarOptionals.UnitScale.Value(), ""))
	}
	// to determine the Content-Type header
	localVarHttpContentTypes := []string{}

	// set Content-Type header
	localVarHttpContentType := selectHeaderContentType(localVarHttpContentTypes)
	if localVarHttpContentType != "" {
		localVarHeaderParams["Content-Type"] = localVarHttpContentType
	}

	// to determine the Accept header
	localVarHttpHeaderAccepts := []string{"application/json"}

	// set Accept header
	localVarHttpHeaderAccept := selectHeaderAccept(localVarHttpHeaderAccepts)
	if localVarHttpHeaderAccept != "" {
		localVarHeaderParams["Accept"] = localVarHttpHeaderAccept
	}
	if ctx != nil {
		// API Key Authentication
		if auth, ok := ctx.Value(ContextAPIKey).(APIKey); ok {
			var key string
			if auth.Prefix != "" {
				key = auth.Prefix + " " + auth.Key
			} else {
				key = auth.Key
			}
			localVarHeaderParams["X-API-KEY"] = key
			
		}
	}
	r, err := a.client.prepareRequest(ctx, localVarPath, localVarHttpMethod, localVarPostBody, localVarHeaderParams, localVarQueryParams, localVarFormParams, localVarFileName, localVarFileBytes)
	if err != nil {
		return localVarReturnValue, nil, err
	}

	localVarHttpResponse, err := a.client.callAPI(r)
	if err != nil || localVarHttpResponse == nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	localVarBody, err := ioutil.ReadAll(localVarHttpResponse.Body)
	localVarHttpResponse.Body.Close()
	if err != nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	if localVarHttpResponse.StatusCode < 300 {
		// If we succeed, return the data, otherwise pass on to decode error.
		err = a.client.decode(&localVarReturnValue, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
		if err == nil { 
			return localVarReturnValue, localVarHttpResponse, err
		}
	}

	if localVarHttpResponse.StatusCode >= 300 {
		newErr := GenericSwaggerError{
			body: localVarBody,
			error: localVarHttpResponse.Status,
		}
		if localVarHttpResponse.StatusCode == 200 {
			var v EcmwfAirQualityForecastPostcodeHourObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		if localVarHttpResponse.StatusCode == 0 {
			var v ErrorObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		return localVarReturnValue, localVarHttpResponse, newErr
	}

	return localVarReturnValue, localVarHttpResponse, nil
}
/*
PostcodesApiService An hour of ECMWF Air Quality Forecast data for a postcode.
Returns a single hour of ECMWF Air Quality Forecast data for a postcode. The provided timestamp must exist within the current valid 10-day forecast range.  **IMPORTANT:** To authenticate to the Weather Source ECMWF Air Quality Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
 * @param ctx context.Context - for authentication, logging, cancellation, deadlines, tracing, etc. Passed from http.Request or context.Background().
 * @param postcode A postcode string.
 * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
 * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required.
 * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* 
 * @param optional nil or *PostcodesApiPostcodesPostcodecountryCodeHoursTimestampGetOpts - Optional Parameters:
     * @param "UnitScale" (optional.String) -  The unit scale for returned values.&lt;br&gt;
@return EcmwfAirQualityForecastPostcodeHourObj
*/

type PostcodesApiPostcodesPostcodecountryCodeHoursTimestampGetOpts struct {
    UnitScale optional.String
}

func (a *PostcodesApiService) PostcodesPostcodecountryCodeHoursTimestampGet(ctx context.Context, postcode string, countryCode string, timestamp time.Time, fields []string, localVarOptionals *PostcodesApiPostcodesPostcodecountryCodeHoursTimestampGetOpts) (EcmwfAirQualityForecastPostcodeHourObj, *http.Response, error) {
	var (
		localVarHttpMethod = strings.ToUpper("Get")
		localVarPostBody   interface{}
		localVarFileName   string
		localVarFileBytes  []byte
		localVarReturnValue EcmwfAirQualityForecastPostcodeHourObj
	)

	// create path and map variables
	localVarPath := a.client.cfg.BasePath + "/postcodes/{postcode},{countryCode}/hours/{timestamp}"
	localVarPath = strings.Replace(localVarPath, "{"+"postcode"+"}", fmt.Sprintf("%v", postcode), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"countryCode"+"}", fmt.Sprintf("%v", countryCode), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"timestamp"+"}", fmt.Sprintf("%v", timestamp), -1)

	localVarHeaderParams := make(map[string]string)
	localVarQueryParams := url.Values{}
	localVarFormParams := url.Values{}

	localVarQueryParams.Add("fields", parameterToString(fields, "csv"))
	if localVarOptionals != nil && localVarOptionals.UnitScale.IsSet() {
		localVarQueryParams.Add("unitScale", parameterToString(localVarOptionals.UnitScale.Value(), ""))
	}
	// to determine the Content-Type header
	localVarHttpContentTypes := []string{}

	// set Content-Type header
	localVarHttpContentType := selectHeaderContentType(localVarHttpContentTypes)
	if localVarHttpContentType != "" {
		localVarHeaderParams["Content-Type"] = localVarHttpContentType
	}

	// to determine the Accept header
	localVarHttpHeaderAccepts := []string{"application/json"}

	// set Accept header
	localVarHttpHeaderAccept := selectHeaderAccept(localVarHttpHeaderAccepts)
	if localVarHttpHeaderAccept != "" {
		localVarHeaderParams["Accept"] = localVarHttpHeaderAccept
	}
	if ctx != nil {
		// API Key Authentication
		if auth, ok := ctx.Value(ContextAPIKey).(APIKey); ok {
			var key string
			if auth.Prefix != "" {
				key = auth.Prefix + " " + auth.Key
			} else {
				key = auth.Key
			}
			localVarHeaderParams["X-API-KEY"] = key
			
		}
	}
	r, err := a.client.prepareRequest(ctx, localVarPath, localVarHttpMethod, localVarPostBody, localVarHeaderParams, localVarQueryParams, localVarFormParams, localVarFileName, localVarFileBytes)
	if err != nil {
		return localVarReturnValue, nil, err
	}

	localVarHttpResponse, err := a.client.callAPI(r)
	if err != nil || localVarHttpResponse == nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	localVarBody, err := ioutil.ReadAll(localVarHttpResponse.Body)
	localVarHttpResponse.Body.Close()
	if err != nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	if localVarHttpResponse.StatusCode < 300 {
		// If we succeed, return the data, otherwise pass on to decode error.
		err = a.client.decode(&localVarReturnValue, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
		if err == nil { 
			return localVarReturnValue, localVarHttpResponse, err
		}
	}

	if localVarHttpResponse.StatusCode >= 300 {
		newErr := GenericSwaggerError{
			body: localVarBody,
			error: localVarHttpResponse.Status,
		}
		if localVarHttpResponse.StatusCode == 200 {
			var v EcmwfAirQualityForecastPostcodeHourObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		if localVarHttpResponse.StatusCode == 0 {
			var v ErrorObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		return localVarReturnValue, localVarHttpResponse, newErr
	}

	return localVarReturnValue, localVarHttpResponse, nil
}
/*
PostcodesApiService A contiguous range of hourly ECMWF Air Quality Forecast data for a postcode.
Returns a contiguous range of hourly ECMWF Air Quality Forecast data for a postcode. The provided timestamps must exist within the current valid 10-day forecast range.  **IMPORTANT:** To authenticate to the Weather Source ECMWF Air Quality Forecast API, each request must contain your API key set to a custom header named &#x60;X-API-KEY&#x60;. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). You may use the evaluation API Key &#x60;C0W60UOFRML47ytbXk4xlLBfv&#x60; to access example locations found in this documentation.  When using the \\\&quot;Try it out\\\&quot; capability in the documentation, the API key should be added by clicking the green \\\&quot;Authorize\\\&quot; button at the top of the page. 
 * @param ctx context.Context - for authentication, logging, cancellation, deadlines, tracing, etc. Passed from http.Request or context.Background().
 * @param postcode A postcode string.
 * @param countryCode An uppercase 2-character [ISO 3166-1 Alpha-2 country code](https://developer.weathersourceapis.com/apis/countries-with-postal-code-support/).
 * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required.
 * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required.
 * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* 
 * @param optional nil or *PostcodesApiPostcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetOpts - Optional Parameters:
     * @param "UnitScale" (optional.String) -  The unit scale for returned values.&lt;br&gt;
@return EcmwfAirQualityForecastPostcodeHourObj
*/

type PostcodesApiPostcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetOpts struct {
    UnitScale optional.String
}

func (a *PostcodesApiService) PostcodesPostcodecountryCodeHoursTimestampStarttimestampEndGet(ctx context.Context, postcode string, countryCode string, timestampStart time.Time, timestampEnd time.Time, fields []string, localVarOptionals *PostcodesApiPostcodesPostcodecountryCodeHoursTimestampStarttimestampEndGetOpts) (EcmwfAirQualityForecastPostcodeHourObj, *http.Response, error) {
	var (
		localVarHttpMethod = strings.ToUpper("Get")
		localVarPostBody   interface{}
		localVarFileName   string
		localVarFileBytes  []byte
		localVarReturnValue EcmwfAirQualityForecastPostcodeHourObj
	)

	// create path and map variables
	localVarPath := a.client.cfg.BasePath + "/postcodes/{postcode},{countryCode}/hours/{timestampStart},{timestampEnd}"
	localVarPath = strings.Replace(localVarPath, "{"+"postcode"+"}", fmt.Sprintf("%v", postcode), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"countryCode"+"}", fmt.Sprintf("%v", countryCode), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"timestampStart"+"}", fmt.Sprintf("%v", timestampStart), -1)
	localVarPath = strings.Replace(localVarPath, "{"+"timestampEnd"+"}", fmt.Sprintf("%v", timestampEnd), -1)

	localVarHeaderParams := make(map[string]string)
	localVarQueryParams := url.Values{}
	localVarFormParams := url.Values{}

	localVarQueryParams.Add("fields", parameterToString(fields, "csv"))
	if localVarOptionals != nil && localVarOptionals.UnitScale.IsSet() {
		localVarQueryParams.Add("unitScale", parameterToString(localVarOptionals.UnitScale.Value(), ""))
	}
	// to determine the Content-Type header
	localVarHttpContentTypes := []string{}

	// set Content-Type header
	localVarHttpContentType := selectHeaderContentType(localVarHttpContentTypes)
	if localVarHttpContentType != "" {
		localVarHeaderParams["Content-Type"] = localVarHttpContentType
	}

	// to determine the Accept header
	localVarHttpHeaderAccepts := []string{"application/json"}

	// set Accept header
	localVarHttpHeaderAccept := selectHeaderAccept(localVarHttpHeaderAccepts)
	if localVarHttpHeaderAccept != "" {
		localVarHeaderParams["Accept"] = localVarHttpHeaderAccept
	}
	if ctx != nil {
		// API Key Authentication
		if auth, ok := ctx.Value(ContextAPIKey).(APIKey); ok {
			var key string
			if auth.Prefix != "" {
				key = auth.Prefix + " " + auth.Key
			} else {
				key = auth.Key
			}
			localVarHeaderParams["X-API-KEY"] = key
			
		}
	}
	r, err := a.client.prepareRequest(ctx, localVarPath, localVarHttpMethod, localVarPostBody, localVarHeaderParams, localVarQueryParams, localVarFormParams, localVarFileName, localVarFileBytes)
	if err != nil {
		return localVarReturnValue, nil, err
	}

	localVarHttpResponse, err := a.client.callAPI(r)
	if err != nil || localVarHttpResponse == nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	localVarBody, err := ioutil.ReadAll(localVarHttpResponse.Body)
	localVarHttpResponse.Body.Close()
	if err != nil {
		return localVarReturnValue, localVarHttpResponse, err
	}

	if localVarHttpResponse.StatusCode < 300 {
		// If we succeed, return the data, otherwise pass on to decode error.
		err = a.client.decode(&localVarReturnValue, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
		if err == nil { 
			return localVarReturnValue, localVarHttpResponse, err
		}
	}

	if localVarHttpResponse.StatusCode >= 300 {
		newErr := GenericSwaggerError{
			body: localVarBody,
			error: localVarHttpResponse.Status,
		}
		if localVarHttpResponse.StatusCode == 200 {
			var v EcmwfAirQualityForecastPostcodeHourObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		if localVarHttpResponse.StatusCode == 0 {
			var v ErrorObj
			err = a.client.decode(&v, localVarBody, localVarHttpResponse.Header.Get("Content-Type"));
				if err != nil {
					newErr.error = err.Error()
					return localVarReturnValue, localVarHttpResponse, newErr
				}
				newErr.model = v
				return localVarReturnValue, localVarHttpResponse, newErr
		}
		return localVarReturnValue, localVarHttpResponse, newErr
	}

	return localVarReturnValue, localVarHttpResponse, nil
}
